/*
 * Copyright 2003-2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codehaus.groovy.tools.stubgenerator

/**
 * Checks that multiline Strings correctly appear within stubs.
 *
 * @author Paul King
 */
class MultilineStringStubTest extends StringSourcesStubTestCase {

    Map<String, String> provideSources() {
        [
            'MyClass.groovy': '''
                class MyClass {
                   public static final String FOO = """Bar\nBaz"""
                }
            ''',

            'Main.java': '''
                public class Main {
                   public void main() {
                      System.out.println(new MyClass().FOO);
                   }
                }
            '''
        ]
    }

    void verifyStubs() {
        def stubSource = stubJavaSourceFor('MyClass')
        assert stubSource.contains('public static final java.lang.String FOO = "Bar\\nBaz";')
    }
}
